/*
  Example allows control of Led Diode connected to pin 3.
  
  To access device:
  1. Download LocalServer and GateHub from Stargate page
  2. Run LocalServer on machine within the same network
  3. Upload program to your board
  4. Run GateHub on machine connected with your board by USB cable
    (if you need to upload your program again, stop GateHub first)

  Before uploading, make sure ledPin on your board is an analog pin
  and is not RX/TX pin (library is using RX/TX to communicate)
*/

#include <SerialGateDevice.h>

int ledPin = 3;

// creating device instance
SerialGateDevice device;
// declaring Gate Value to store current LED setting
GateInt led = device.factory.createInt("input");

void setup() {
  pinMode(ledPin, OUTPUT);
  // setting device name
  device.setName("Basic LED");
  // setting Gate Value name
  led.setName("Brightness");
  // setting allowed range
  led.setMin(0);
  led.setMax(255);
  // setting callback to be invoked if Gate Value was changed remotely
  led.onRemoteChange = onLedChange;
  // starting device
  device.start();
}

void loop() {
  // running device internal processes
  device.loop();
}

void onLedChange() {
  analogWrite(ledPin, led.getValue());
}