/*
  Modified Basic LED example that uses ServerStorage to persist latest LED
  setting and restore it on reboot.
  
  Usage:
    1. Upload sketch to your board - led should be off
    2. Open User Interface in browser and set LED brightness to some value
    3. Reboot your board - LED brightness should be restored
*/

#include <SerialGateDevice.h>

int ledPin = 1;

SerialGateDevice device;
GateInt led = device.factory.createInt("input");

// flag to indicate if state should be restored
bool initLed = true;
// key under which LED state will be stored
String STORAGE_KEY = "ledState";

void setup() {
  pinMode(ledPin, OUTPUT);
  device.setName("ServerStorage");
  led.setName("Brightness");
  led.setMin(0);
  led.setMax(255);
  led.onRemoteChange = onLedChange;
  device.start();
}

void loop() {
  // checking if LED state should be initialized
  if (initLed && device.isReady()) {
    initLed = false;
    // fetching last known LED state from ServerStorage
    device.serverStorage.get(STORAGE_KEY, initializeLed);
  }
  device.loop();
}

void onLedChange() {
  analogWrite(ledPin, led.getValue());
  device.serverStorage.set(STORAGE_KEY, String(led.getValue()));
}

void initializeLed(String storedValue) {
  // checking if response is not empty
  if (storedValue.length() > 0) {
    // parsing received string response to integer value
    int value = storedValue.toInt();
    // setting LED state to the one received from ServerStorage
    analogWrite(ledPin, value);
    led.setValue(value);
  }
}
